@@ -31,10 +31,8 @@ import numpy as np
 import pandas as pd
 
 # local imports
-from .simulation import (
-    biased_simulation_from_binary_state,
-    simulate_from_boolean_trajectory,
-)
+from .simulation import biased_simulation_from_binary_state
+from .utils.stream_helpers import simulate_from_boolean_trajectory, SampleCountSpec
 from .binarization import _binarize as binarization_binarize
 
 # R source code locations :
@@ -137,9 +135,9 @@ class scBoolSeq(object):
                     [
                         f"Parameter `{_parameter_name}` has {_na_count} NaN entries",
                         "this will cause undefined behaviour when computing criteria",
-                        "or simulating expression data."
-                        "Please verify that all entries of your dataframe",
-                        "are valid numerical entries.",
+                        "or simulating expression data. "
+                        "Please verify that all entries of your dataframe ",
+                        "are valid numerical entries. ",
                     ]
                 )
             )
@@ -197,7 +195,7 @@ class scBoolSeq(object):
     def __repr__(self):
         _attrs = (
             f"has_data={self._has_data}",
-            f"is_trained={self._is_trained}",
+            f"can_binarize={self._is_trained}",
             f"can_simulate={self._can_simulate}",
         )
         return f"scBoolSeq({', '.join(_attrs)})"
@@ -666,8 +664,7 @@ class scBoolSeq(object):
     def simulate_with_metadata(
         self,
         binary_df: pd.DataFrame,
-        n_samples_per_state: int = 1,
-        n_threads: Optional[int] = None,
+        n_samples_per_state: SampleCountSpec = 1,
         rng_seed: Optional[int] = None,
     ) -> Tuple[pd.DataFrame, pd.DataFrame]:
         """
@@ -677,21 +674,16 @@ class scBoolSeq(object):
         and a metadata DataFrame which can be used to apply trajectory
         reconstruction methods.
 
-        wrapper for scBoolSeq.simulation.simulate_from_boolean_trajectory
+        wrapper for scBoolSeq.utils.stream_helpers.simulate_from_boolean_trajectory
         """
         if not self._can_simulate:
             raise AttributeError("Call .simulation_fit() first")
         self._check_df_contains_no_nan(_df=binary_df, _parameter_name="binary_df")
-        n_threads = (
-            min(abs(n_threads), multiprocessing.cpu_count())
-            if n_threads
-            else multiprocessing.cpu_count()
-        )
+
         return simulate_from_boolean_trajectory(
             boolean_trajectory_df=binary_df,
             criteria_df=self.simulation_criteria,
             n_samples_per_state=n_samples_per_state,
-            n_threads=n_threads,
             rng_seed=rng_seed,
         )
 
